<?php

namespace Drupal\cache_review\Controller;

use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\Core\Link;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class with demo cache pages.
 */
class CacheReviewController extends ControllerBase implements TrustedCallbackInterface {

  /**
   * The extension path resolver.
   *
   * @var \Drupal\Core\Extension\ExtensionPathResolver
   */
  protected $extensionPathResolver;

  /**
   * The route provider to load routes by name.
   *
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * The block manager.
   *
   * @var \Drupal\Core\Block\BlockManagerInterface
   */
  protected $blockManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Construct for CacheReview object.
   *
   * @param \Drupal\Core\Extension\ExtensionPathResolver|null $extension_path_resolver
   *   The extension path resolver.
   * @param \Drupal\Core\Routing\RouteProviderInterface $route_provider
   *   The route provider.
   * @param \Drupal\Core\Block\BlockManagerInterface $block_manager
   *   The block manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   */
  public function __construct(ExtensionPathResolver $extension_path_resolver = NULL, RouteProviderInterface $route_provider, BlockManagerInterface $block_manager, AccountInterface $current_user, RendererInterface $renderer) {
    if (!$extension_path_resolver) {
      @trigger_error('Calling LibraryDiscoveryParser::__construct() without the $extension_path_resolver argument is deprecated in drupal:9.3.0 and is required in drupal:10.0.0. See https://www.drupal.org/node/2940438', E_USER_DEPRECATED);
      $extension_path_resolver = \Drupal::service('extension.path.resolver');
    }
    $this->extensionPathResolver = $extension_path_resolver;
    $this->routeProvider = $route_provider;
    $this->blockManager = $block_manager;
    $this->currentUser = $current_user;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('extension.path.resolver'),
      $container->get('router.route_provider'),
      $container->get('plugin.manager.block'),
      $container->get('current_user'),
      $container->get('renderer'),
    );
  }

  /**
   * Page with links.
   */
  public function page() {
    $build['links'] = [];
    $build['links']['#markup'] = '';

    $path = $this->extensionPathResolver->getPath('module', 'cache_review');
    $routingFilePath = DRUPAL_ROOT . '/' . $path . '/cache_review.routing.yml';
    $routingFileContents = file_get_contents($routingFilePath);
    $results = Yaml::decode($routingFileContents);
    foreach ($results as $id => $result) {
      $routeName = $id;
      $route = $this->routeProvider->getRouteByName($routeName);

      $route_title = $route->getDefault('_title');
      if ($routeName === 'cache_review.page') {
        continue;
      }

      $build['links']['#markup'] .= Link::createFromRoute($route_title, $routeName, [], [
        'absolute' => TRUE,
        'https' => TRUE,
        'attributes' => ['target' => '_blank'],
      ])->toString() . '<br>';
    }

    return $build;
  }

  /**
   * Cached page with different max-age for items.
   */
  public function cachedMaxAge() {
    $date = '<span class="date">' . date('H:i:s') . ' </span>';

    $build = [
      'permanent' => [
        '#markup' => ' (01) ' . $date . $this->randomText() . '<br><b>Options:' . json_encode([
          'max-age' => Cache::PERMANENT,
          'tags' => ['user:1'],
        ]) . '</b><br>',
        '#cache' => [
          'max-age' => Cache::PERMANENT,
          'tags' => ['user:1'],
        ],
      ],
      'parent_item' => [
        'child_item_one' => [
          '#markup' => ' (02) ' . $date . $this->randomText() . '<br><b>Options:' . json_encode(['max-age' => 20]) . '</b><br>',
          '#cache' => [
            'max-age' => 20,
          ],
        ],
        'child_item_two' => [
          '#markup' => ' (03) ' . $date . $this->randomText() . '<br><b>Options:' . json_encode([
            'max-age' => 10,
            'keys' => ['demo_child'],
          ]) . '</b><br>',
          '#cache' => [
            'max-age' => 10,
            'keys' => ['demo_child'],
          ],
        ],
      ],
      'contexts_url' => [
        '#markup' => ' (04) ' . $date . $this->randomText() . '<br><b>Options:' . json_encode(['contexts' => ['url.query_args']]) . '</b><br>',
        '#cache' => [
          'contexts' => ['url.query_args'],
        ],
      ],
    ];
    return $build;
  }

  /**
   * Page with lazy, context and keys.
   */
  public function uncacheableLazyKeys() {
    $date = '<span class="date">' . date('H:i:s') . ' </span>';
    $context = 'user';
    $block = $this->blockManager->createInstance('cache_demo_block_no_lazy')
      ->build($context);
    $block_render = $this->renderer->render($block);

    $build = [
      'block_uncached' => [
        '#markup' => $block_render . '<br>',
      ],
      'permanent' => [
        '#markup' => ' (1) ' . $date . $this->randomText() . '<br><b>Options:' . json_encode(['max-age' => Cache::PERMANENT]) . '</b><br>',
        '#cache' => [
          'max-age' => Cache::PERMANENT,
        ],
      ],
      'contexts_url' => [
        '#markup' => ' (2) ' . $date . $this->randomText() . '<br><b>Options:' . json_encode([
          'contexts' => ['user', 'url.query_args'],
          'keys' => ['demo_contexts_url'],
        ]) . '</b><br>',
        '#cache' => [
          'contexts' => ['user', 'url.query_args'],
          'keys' => ['demo_contexts_url'],
        ],
      ],
      'contexts_url_param' => [
        '#markup' => ' (3) ' . $date . $this->randomText() . '<br><b>Options:' . json_encode([
          'contexts' => ['url.query_args:test'],
          'keys' => ['demo_contexts_url_test'],
        ]) . '</b><br>',
        '#cache' => [
          'contexts' => ['url.query_args:test'],
          'keys' => ['demo_contexts_url_test'],
        ],
      ],
      'lazy_contexts_session' => [
        '#cache' => [
          'contexts' => ['session'],
        ],
        '#lazy_builder' => [static::class . '::lazyBuilderContext', []],
      ],
      'lazy_create_placeholder' => [
        '#lazy_builder' => [static::class . '::lazyBuilderPlaceholder', []],
        '#create_placeholder' => TRUE,
      ],
    ];
    return $build;
  }

  /**
   * Lazy builder with context.
   */
  public static function lazyBuilderContext() {
    $date = '<span class="date">' . date('H:i:s') . ' </span>';
    return [
      '#markup' => ' (4) ' . $date . 'This item has context <b>session</b> and <b>#lazy_builder</b> options. As result - item is not cached.' . '<br>',
    ];
  }

  /**
   * Lazy builder with create_placeholder options.
   */
  public static function lazyBuilderPlaceholder() {
    $date = '<span class="date">' . date('H:i:s') . ' </span>';
    return [
      '#markup' => ' (5) ' . $date . 'This item has <b>#lazy_builder</b> and <b>#create_placeholder</b> options. Item is not cached.' . '<br>',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return [
      'lazyBuilderContext',
      'lazyBuilderPlaceholder',
    ];
  }

  /**
   * Cached page with lazy items.
   */
  public function cachedLazy() {
    $date = '<span class="date">' . date('H:i:s') . ' </span>';

    $block = $this->blockManager->createInstance('cache_review_block_lazy')->build();

    $build = [
      'block_lazy' => $block,
      'permanent' => [
        '#markup' => ' (11) ' . $date . $this->randomText() . '<br><b>Options:' . json_encode([
          'max-age' => ['-1'],
          'keys' => ['keys_permanent_test'],
        ]) . '</b><br>',
        '#cache' => [
          'max-age' => Cache::PERMANENT,
          'keys' => ['keys_permanent_test'],
        ],
      ],
      'parent_item' => [
        'child_item' => [
          '#markup' => ' (22) ' . $date . $this->randomText() . '<br><b>Options: -- </b><br>',
        ],
      ],
      'contexts_url' => [
        '#markup' => ' (33) ' . $date . $this->randomText() . '<br><b>Options:' . json_encode(['contexts' => ['url.query_args']]) . '</b><br>',
        '#cache' => [
          'contexts' => ['url.query_args'],
        ],
      ],
    ];

    // Calling Lazy Builder via service.
    $account = $this->currentUser;
    $build['lazy_item'] = [
      '#lazy_builder' => [
        'cache_review.lazy_builder:lazyBuilderCustom',
        [$account->id()],
      ],
      '#cache' => [
        'contexts' => ['user'],
      ],
    ];

    return $build;
  }

  /**
   * Helper to get some random text.
   */
  private function randomText() {
    $str = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Praesent nec ante auctor, feugiat sem nec, mollis mi Vestibulum leo quam, facilisis non porttitor et';
    $words = str_word_count($str, 1);
    shuffle($words);
    return implode(' ', $words);
  }

}
