<?php

namespace Drupal\cache_review\Render;

use Drupal\Core\Cache\CacheFactoryInterface;
use Drupal\Core\Cache\Context\CacheContextsManager;
use Drupal\Core\Render\Markup;
use Drupal\Core\Render\PlaceholderGeneratorInterface;
use Drupal\Core\Render\PlaceholderingRenderCache;
use Drupal\Core\Routing\CurrentRouteMatch;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Decorate RenderCache service.
 *
 * Adding mark for cached and non-cached elements.
 */
class RenderCacheReview extends PlaceholderingRenderCache {
  /**
   * Original PlaceholderingRenderCache.
   *
   * @var \Drupal\Core\Render\PlaceholderingRenderCache
   */
  protected $placeholderingRenderCache;

  /**
   * The current route match service.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * The renderer configuration array.
   *
   * @var array
   */
  protected array $rendererConfig;

  /**
   * Construct RenderCacheReview object.
   *
   * @param \Drupal\Core\Render\PlaceholderingRenderCache $original_placeholdering_render_cache
   *   The original placeholdering_render_cache.
   * @param array $renderer_config
   *   The renderer configuration array.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $current_route_match
   *   The current route match service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Cache\CacheFactoryInterface $cache_factory
   *   The cache factory.
   * @param \Drupal\Core\Cache\Context\CacheContextsManager $cache_contexts_manager
   *   The cache contexts manager.
   * @param \Drupal\Core\Render\PlaceholderGeneratorInterface $placeholder_generator
   *   The placeholder generator.
   */
  public function __construct(
    PlaceholderingRenderCache $original_placeholdering_render_cache,
    array $renderer_config,
    CurrentRouteMatch $current_route_match,
    RequestStack $request_stack,
    CacheFactoryInterface $cache_factory,
    CacheContextsManager $cache_contexts_manager,
    PlaceholderGeneratorInterface $placeholder_generator,
  ) {
    $this->placeholderingRenderCache = $original_placeholdering_render_cache;
    $this->rendererConfig = $renderer_config;
    $this->currentRouteMatch = $current_route_match;
    parent::__construct($request_stack, $cache_factory, $cache_contexts_manager, $placeholder_generator);
  }

  /**
   * {@inheritdoc}
   */
  public function get(array $elements) {
    // Part form PlaceholderingRenderCache->get().
    // @todo remove this check when https://www.drupal.org/node/2367555 lands.
    if (!$this->requestStack->getCurrentRequest()->isMethodCacheable()) {
      return FALSE;
    }
    if (isset($elements['#create_placeholder']) && $elements['#create_placeholder'] === FALSE) {
      $cached_placeholder_result = $this->getFromPlaceholderResultsCache($elements);
      if ($cached_placeholder_result !== FALSE) {
        return $cached_placeholder_result;
      }
    }

    // Call overridden RenderCache->get().
    $cached_element = $this->parentGet($elements);

    if ($cached_element === FALSE) {
      return FALSE;
    }
    else {
      if ($this->placeholderGenerator->canCreatePlaceholder($elements) && $this->placeholderGenerator->shouldAutomaticallyPlaceholder($cached_element)) {
        return $this->createPlaceholderAndRemember($cached_element, $elements);
      }
      return $cached_element;
    }

  }

  /**
   * Overridden Part from RenderCache->get().
   */
  private function parentGet(array $elements) {
    if (!$this->requestStack->getCurrentRequest()->isMethodCacheable() || !$cid = $this->createCacheID($elements)) {
      return FALSE;
    }
    $bin = $elements['#cache']['bin'] ?? 'render';

    if (!empty($cid) && ($cache_bin = $this->cacheFactory->get($bin)) && $cache = $cache_bin->get($cid)) {
      $cached_element = $cache->data;

      if (isset($cached_element['#cache_redirect'])) {
        return $this->parentGet($cached_element);
      }

      if ($current_route = $this->currentRouteMatch->getRouteObject()) {
        $is_admin = (bool) $current_route->getOption('_admin_route');
        // Add cache info for non-admin routes only.
        if (!$is_admin) {
          // Add mark to cached element.
          $markup_string = (string) $cached_element['#markup'];
          $catched_condition = '';
          if (!empty($markup_string)) {

            $apc = $this->rendererConfig['auto_placeholder_conditions'];
            if ($cached_element['#cache']['max-age'] == $apc['max-age'] ||
              array_intersect($cached_element['#cache']['contexts'], $apc['contexts']) ||
              array_intersect($cached_element['#cache']['tags'], $apc['tags'])) {
              $catched_condition = ' cache_caught';
            }

            $prefix = '<div class="cached-wrapper">';
            $suffix = '</div>';
            $options_data = 'contexts: ' . json_encode(array_values($cached_element['#cache']['contexts'])) . '<br>' .
              'max-age: ' . json_encode($cached_element['#cache']['max-age']) . '<br>' .
              'tags: ' . json_encode(array_values($cached_element['#cache']['tags']));
            $options = '<span class="cache_tooltiptext">' . $options_data . '</span>';
            $cache_mark = '<div class="cached-item cache_tooltip' . $catched_condition . '">C' . $options . '</div>';
            $cached_element['#markup'] = Markup::create($prefix . $cache_mark . $markup_string . $suffix);
          }
        }
      }

      return $cached_element;
    }
    return FALSE;
  }

}
