<?php

namespace Drupal\cache_review\Render;

use Drupal\Core\Controller\ControllerResolverInterface;
use Drupal\Core\Render\ElementInfoManagerInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Render\PlaceholderGeneratorInterface;
use Drupal\Core\Render\RenderCacheInterface;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Theme\ThemeManagerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Decorates core's Renderer to add mark to placeholders.
 */
class RendererCacheReview extends Renderer {

  /**
   * The original Renderer.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * The current route match service.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * Construct RendererCacheReview object.
   *
   * @param \Drupal\Core\Render\Renderer $original_renderer
   *   The original renderer.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $current_route_match
   *   The current route match service.
   * @param \Drupal\Core\Controller\ControllerResolverInterface $controller_resolver
   *   The controller resolver.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $theme
   *   The theme manager.
   * @param \Drupal\Core\Render\ElementInfoManagerInterface $element_info
   *   The element info.
   * @param \Drupal\Core\Render\PlaceholderGeneratorInterface $placeholder_generator
   *   The placeholder generator.
   * @param \Drupal\Core\Render\RenderCacheInterface $render_cache
   *   The render cache service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param array $renderer_config
   *   The renderer configuration array.
   */
  public function __construct(Renderer $original_renderer, CurrentRouteMatch $current_route_match, ControllerResolverInterface $controller_resolver, ThemeManagerInterface $theme, ElementInfoManagerInterface $element_info, PlaceholderGeneratorInterface $placeholder_generator, RenderCacheInterface $render_cache, RequestStack $request_stack, array $renderer_config) {
    $this->renderer = $original_renderer;
    $this->currentRouteMatch = $current_route_match;
    parent::__construct($controller_resolver, $theme, $element_info, $placeholder_generator, $render_cache, $request_stack, $renderer_config);
  }

  /**
   * {@inheritdoc}
   */
  protected function doRender(&$elements, $is_root_call = FALSE) {
    // Set not-cached mark to add cache options later.
    if (isset($elements['#cache']) && !isset($elements['#cache']['keys'])) {
      $not_cached = TRUE;
    }

    $result = parent::doRender($elements, $is_root_call);

    // Return if nothing to visualize.
    if ($result === '') {
      return '';
    }

    // Add cache info for non-admin routes only.
    if ($current_route = $this->currentRouteMatch->getRouteObject()) {
      $is_admin = (bool) $current_route->getOption('_admin_route');
    }

    // Add cache data for lazy built elements.
    if (empty($is_admin) && !empty($elements['#attached']['placeholders']) && count($elements['#attached']['placeholders']) === 1) {
      $lazy_cache = '';

      if (!empty(reset($elements['#attached']['placeholders'])['#cache'])) {
        $lazy_cache = '<b>Lazy options:</b> ' . json_encode(reset($elements['#attached']['placeholders'])['#cache']) . '<br>';
      }
      $markup_string = (string) $elements['#markup'];
      if (!empty($markup_string)) {
        $prefix = '<div class="lazy-wrapper">';
        $suffix = '</div>';
        $options_data = '<b>Item options:</b><br> contexts: ' . json_encode(array_values($elements['#cache']['contexts'])) . '<br>' .
          'max-age: ' . json_encode($elements['#cache']['max-age']) . '<br>' .
          'tags: ' . json_encode(array_values($elements['#cache']['tags']));
        $options = '<span class="cache_tooltiptext">' . $lazy_cache . $options_data . '</span>';
        $cache_mark = '<div class="lazy-item cache_tooltip">L' . $options . '</div>';
        $elements['#markup'] = Markup::create($prefix . $cache_mark . $markup_string . $suffix);
      }
    }

    // Add cache data for non-cached (separately) elements.
    if (empty($is_admin) && !empty($not_cached)) {
      $markup_string = (string) $elements['#markup'];
      $catched_condition = '';
      if (!empty($markup_string)) {

        $apc = $this->rendererConfig['auto_placeholder_conditions'];
        if (array_intersect($elements['#cache']['contexts'], $apc['contexts']) ||
          array_intersect($elements['#cache']['tags'], $apc['tags']) ||
          $elements['#cache']['max-age'] == $apc['max-age']) {
          $catched_condition = ' cache_caught';
        }

        $prefix = '<div class="base-wrapper">';
        $suffix = '</div>';
        $options_data = '<b>Item options:</b><br> contexts: ' . json_encode(array_values($elements['#cache']['contexts'])) . '<br>' .
          'max-age: ' . json_encode($elements['#cache']['max-age']) . '<br>' .
          'tags: ' . json_encode(array_values($elements['#cache']['tags']));
        $options = '<span class="cache_tooltiptext">' . $options_data . '</span>';
        $cache_mark = '<div class="base-item cache_tooltip' . $catched_condition . '">N' . $options . '</div>';
        $elements['#markup'] = Markup::create($prefix . $cache_mark . $markup_string . $suffix);
      }
    }

    return $elements['#markup'];
  }

}
