<?php

namespace Drupal\cache_review\StackMiddleware;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\dynamic_page_cache\EventSubscriber\DynamicPageCacheSubscriber;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Show cache variables before page load.
 */
class CacheReview implements HttpKernelInterface {

  /**
   * The wrapped HTTP kernel.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The renderer configuration array.
   *
   * @var array
   */
  protected $rendererConfig;

  /**
   * The current route match service.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a PageCache object.
   *
   * @param \Symfony\Component\HttpKernel\HttpKernelInterface $http_kernel
   *   The decorated kernel.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param array $renderer_config
   *   The renderer configuration array.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $current_route_match
   *   The current route match service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(HttpKernelInterface $http_kernel, ModuleHandlerInterface $module_handler, array $renderer_config, CurrentRouteMatch $current_route_match, RequestStack $request_stack) {
    $this->httpKernel = $http_kernel;
    $this->moduleHandler = $module_handler;
    $this->rendererConfig = $renderer_config;
    $this->currentRouteMatch = $current_route_match;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public function handle(Request $request, $type = self::MASTER_REQUEST, $catch = TRUE): Response {
    $message = '';
    $response = $this->httpKernel->handle($request, $type, $catch);
    $content = $response->getContent();

    // Do not show cache info for admin routes.
    // Fix for request for Anonymous according to
    // https://www.drupal.org/project/drupal/issues/2701575.
    $current_request = $this->requestStack->getCurrentRequest();
    if (!empty($current_request) && !empty($current_route = $this->currentRouteMatch->getRouteObject())) {
      $is_admin = (bool) $current_route->getOption('_admin_route');
      if (!empty($is_admin)) {
        return $response;
      }
    }

    // Get APC values.
    $apc = $this->rendererConfig['auto_placeholder_conditions'];

    // Get IPC and DPC cache values.
    if (!$this->moduleHandler->moduleExists('page_cache')) {
      $message .= "The module page_cache disabled <br>";
    }
    else {
      $ipc = $response->headers->get('X-Drupal-Cache');
    }

    if (!$this->moduleHandler->moduleExists('dynamic_page_cache')) {
      $message .= "The module dynamic_page_cache disabled <br>";
    }
    else {
      $dpc_header = DynamicPageCacheSubscriber::HEADER;
      $dpc = $response->headers->get(DynamicPageCacheSubscriber::HEADER);
    }
    if (!empty($dpc) && $response->isSuccessful()) {
      $ipc_dpc_output = sprintf("IPC response status 'X-Drupal-Cache': %s<br>
        DPC response status '%s': %s", $ipc, $dpc_header, $dpc);
      $content = $message . $ipc_dpc_output . $content;
    }

    if (!empty($apc)) {
      $apc_text = sprintf('Current auto_placeholder_conditions: %s <br>', json_encode($apc));
      $content = $apc_text . $content;
    }
    $time = date('H:i:s');
    $info = 'Designations: <b>C</b> - item cached separately, <b>L</b> - from Lazy builder, <b>N</b> - normal item cache properties<br>
            <span class="cache_caught">C, N</span> - if in red, then this item has cache options matched to auto_placeholder_conditions.<br>';
    $content = '<div class="cache-designition"><b>' . $time . '</b><br>' . $info . $content . '</div>';

    $response->setContent($content);

    return $response;
  }

}
